package ch.hslu.appe.order.persistency;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Random;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import ch.hslu.appe.inventory.model.Product;
import ch.hslu.appe.inventory.persistency.InventoryPersistencyImpl;
import ch.hslu.appe.order.model.Bill;
import ch.hslu.appe.order.model.BillState;
import ch.hslu.appe.order.model.BillTyp;
import ch.hslu.appe.order.model.DeliverState;
import ch.hslu.appe.order.model.Order;
import ch.hslu.appe.order.model.OrderPosition;
import ch.hslu.appe.order.model.OrderState;
import ch.hslu.appe.usermanagement.model.Address;
import ch.hslu.appe.usermanagement.model.User;
import ch.hslu.appe.usermanagement.persistency.AddressPersistencyImpl;
import ch.hslu.appe.usermanagement.persistency.UserPersistencyImpl;

/**
 * OrderPersistency Implementation JUnit Tests.
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class OrderPersistencyImplTest {

    private OrderPersistencyImpl persistency;
    private UserPersistencyImpl persistencyUser;
    private AddressPersistencyImpl persistencyAddr;
    private InventoryPersistencyImpl persistencyProduct;

    private Order testOrder;
    private Date orderDate;
    private Date deliverDate;

    /**
     * Setup persistence.
     */
    @Before
    public final void setUp() {
        persistency = new OrderPersistencyImpl();
        persistencyUser = new UserPersistencyImpl();
        persistencyAddr = new AddressPersistencyImpl();
        persistencyProduct = new InventoryPersistencyImpl();

        final Random rand = new Random();

        // DateFormat dateOnly = DateFormat.getDateInstance(DateFormat.MEDIUM);
        orderDate = new GregorianCalendar(2014, 4, 16).getTime(); // Month value is 0-based
        deliverDate = new GregorianCalendar(2014, 4, 21).getTime(); // Month value is 0-based
        // dateOnly.format(orderDate)

        final User customer = persistencyUser.getUser(6);
        final User salesUser = persistencyUser.getUser(1);
        final Address deliverAddr = persistencyAddr.getAddress(1);
        final List<OrderPosition> newOrderPositions = new ArrayList<OrderPosition>();
        final List<Product> allProducs = persistencyProduct.getProductsActive();
        // add random 0..20 products to the test order
        for (int i = 0; i < rand.nextInt(20); i++) {
            OrderPosition orderPos = new OrderPosition();
            Product orderProduct;
            // get random product
            orderProduct = allProducs.get(rand.nextInt(allProducs.size() - 1));
            orderPos.setPosition(i);
            orderPos.setProduct(orderProduct);
            orderPos.setPrice(orderProduct.getPrice());
            orderPos.setAmount(rand.nextInt(100)); // random amount 0..100
            // add OrderPosition to the newOrderPos List
            newOrderPositions.add(orderPos);
            // System.out.println("# addOrderPos " + orderPos);
        }

        final Bill bill = new Bill();
        bill.setTyp(BillTyp.BILL);
        bill.setDunningCosts(0);
        bill.setBillAddress(deliverAddr);
        bill.setInvoiceDate(orderDate);
        bill.setPaymentDueDate(deliverDate);
        bill.setState(BillState.OPEN);

        final Bill billReminder = new Bill();
        billReminder.setTyp(BillTyp.REMINDER);
        billReminder.setDunningCosts(99);
        billReminder.setBillAddress(deliverAddr);
        billReminder.setInvoiceDate(orderDate);
        billReminder.setPaymentDueDate(deliverDate);
        billReminder.setState(BillState.OPEN);

        final List<Bill> bills = new ArrayList<Bill>();
        bills.add(bill);
        bills.add(billReminder);

        testOrder = new Order();
        testOrder.setNumber(90000 + rand.nextInt(1000));
        testOrder.setOrderState(OrderState.OPEN);
        testOrder.setPayState(BillState.OPEN);
        testOrder.setOrderDate(orderDate);
        testOrder.setCustomer(customer);
        testOrder.setSales(salesUser);
        testOrder.setDeliverAddress(deliverAddr);
        testOrder.setDeliverCost(25);
        testOrder.setDeliverDate(deliverDate);
        testOrder.setDeliverState(DeliverState.NONE);

        // Add Products (OrderPosition)
        testOrder.setOrderPositions(newOrderPositions);

        // Add Bills
        testOrder.setBills(bills);

        // Save Order
        persistency.createOrder(testOrder);
    }

    /**
     * CleanUp after the tests.
     */
    @After
    public final void tearDown() {
        // Delete the inserted order
        persistency.deleteOrderByNumber(testOrder.getNumber());
    }

    /**
     * Test getBills() and checks if a list returns.
     */
    @Test
    public final void testGetBills() {
        // TODO Generate TestData !
        final List<Bill> bills = persistency.getBillFromOrder(101);

        // System.out.println("# testGetBills: " + bills.size() + " loaded...");
        Assert.assertTrue(bills.size() > 0);
    }

    /**
     * Test getNextOrderNumber() - should be > 0 ;-).
     */
    @Test
    public final void testGetNextOrderNumber() {
        final int newOrderNumber = persistency.getNextOrderNumber();
        System.out.println("NextOrderNumber: " + newOrderNumber);
        Assert.assertTrue(newOrderNumber > 0);
    }

    /**
     * Test getOrder() and verify all values.
     */
    @Test
    public final void testGetOrder() {

        // TODO Test Data generieren !

        // Get Order by OrderNumber
        final Order order = persistency.getOrder(testOrder.getNumber());

        final DateFormat dateOnly = DateFormat.getDateInstance(DateFormat.MEDIUM);

        // expected, actual
        Assert.assertEquals(testOrder.getId(), order.getId());
        Assert.assertEquals(testOrder.getNumber(), order.getNumber());
        Assert.assertEquals(testOrder.getPayState(), order.getPayState());
        Assert.assertEquals(testOrder.getOrderState(), order.getOrderState());
        Assert.assertEquals(dateOnly.format(testOrder.getOrderDate()), dateOnly.format(order.getOrderDate()));
        Assert.assertEquals(testOrder.getDeliverAddress().getId(), order.getDeliverAddress().getId());
        Assert.assertEquals(testOrder.getDeliverState(), order.getDeliverState());
        Assert.assertEquals(dateOnly.format(testOrder.getDeliverDate()), dateOnly.format(order.getDeliverDate()));
        Assert.assertEquals(testOrder.getDeliverCost(), order.getDeliverCost(), 0.1);
        Assert.assertEquals(testOrder.getCustomer().getId(), order.getCustomer().getId());
        Assert.assertEquals(testOrder.getSales().getId(), order.getSales().getId());
    }
}
